import xmlrpclib
from SimpleXMLRPCServer import SimpleXMLRPCServer
import thread
import time

#wrapper for static functions. see http://code.activestate.com/recipes/52304/
class Callable:
    def __init__(self, anycallable):
        self.__call__ = anycallable


class NetworkTools(SimpleXMLRPCServer):
	"""
	This is a set of network tools, the main part being a SERVICE REGISTRY
	Basis of that is a list of (machinename,port number) pairs, that can be looked up
	via a key. The key is the name of a service and the pair indicates where
	to get access to that service via XMLRPC.
	
	This avoids hard coding of port numbers and machine names in the code,
	making everything more flexible

	Launch a server by creating an instance on the therapist machine, so
	
import Network.NetworkTools
a=Network.NetworkTools.NetworkTools()

	simple test if it works:
Network.NetworkTools.NetworkTools.getInfo("hello",True,10)

	This service is accessible via port 8001 on the therapist machine.
	On other machines you should not crate an instance but directly call the 
	Network.NetworkTools.getSocket() and registerService calls
	
	then you can start another python program on another machine and run for example
	
	import xmlrpclib
	proxy = xmlrpclib.ServerProxy("http://urk.twi.tudelft.nl:8001/")
	newportnr=proxy.getSocketNoWait('mynewportname')
	
	registring a function is done for you when you use our XMLRPCServer.run().
	Under the hood it does this:
	
	registerService(self,name,info)
	
	but as an XMLRPCServer user you use
	
	import Network.XMLRPCServer
	s=Network.XMLRPCServer.XMLRPCServer("hallo",("localhost",8010))
	def test(self): 
		print "hallo" # this is a test function provided by our service
		return 1 # needed as XMLRPC does not allow empty returns....
	s.register_function(test,"test") # add the function to our server
	s.run() # run server. This registers service with this NetworkTool server
	
	
	Now you can ask the NetworkTools which services have been registered:
	proxy = xmlrpclib.ServerProxy("http://localhost:8001/")
	proxy.getServices()
	
	which now returns {'hallo': ['localhost', 8010]}
	
	And now you can call the registered test() function inside the new service with
	
	import xmlrpclib
	proxy = xmlrpclib.ServerProxy("http://localhost:8010/")
	proxy.test(1)
	
	This will 
	1: print "hallo" in the window running our XMLRPCServer on port 8010
	2: return and print 1 on the machine on which we called proxy.test(1)
	
	
	@author W.Pasman 13aug09
	
	TODO introduce a way to get free ports on a computer. 
	TODO unregister, etc? Without that we can not really restart properly
	
	
	DISCUSSION. Why do it this way instead of settings file?
	With settings file, you would have to copy the settings file to all computers when things change.
	This way you only need to inform all computers about the location of the NetworkTools.
	"""

	
	server_running=False
	assignedsockets=dict()
	lock=thread.allocate_lock() # lock for access to service calls.
	
	# proxy is the xmlrpc port to the central NetworkTools instantiation.
	proxy=0					# will be set when needed

	# @param sockets is an array of integers that we can use as incoming (server) socket numbers
	def __init__(self):
		thread.start_new_thread(self.runtheserver,(self,2)) # start the server thread
	
	
	
	# this is the server thread
	# @param is unused at this moment.
	def runtheserver(self,*param):
		if self.server_running:
			raise Exception("server already running!")
		self.server_running=True
		print param
		SimpleXMLRPCServer.__init__(self,NetworkTools.getInfoServerAddressAndPort())
		print "Networktools listening on port 8001..."
		self.register_function(self.getInfoNoWait, "getInfoNoWait")
		self.register_function(self.registerService, "registerService")	
		self.register_function(self.getServices, "getServices")	
		
		self.serve_forever()
		
	def serve_forever(self):
		while self.server_running:
			self.handle_request()
		# TODO This seems not enough to free up the socket.
		print "NetworkTools service was stopped"

	def stop(self):
		if not self.server_running:
			raise Exception("server is not running")
		self.server_running=False
		
	# @return the socket associated with given name on THIS MACHINE
	# if name not yet known we associate a still free socket with it and return its id
	# @param servicename is a string representing a service. 
	# @returns -1 if no sockets available
	def getInfoNoWait(self,servicename):
		self.lock.acquire()
		newsocket=-1
		try:
			newsocket=self.assignedsockets[servicename]
		except KeyError:
			pass # return -1
		finally: # this is always run, even when exception occurs.
			self.lock.release()
		return newsocket
	
	
	# @param name is the key under which given service is stored. usually patientid/servicename
	# @param info is the (machinename, port number) under which XMLRCP calls can be made for that service
	# TODO we need to make port number assignment more automatic I think
	def registerService(self,name,info):
		# TODO check that info is really of form (machinename, port number)
		self.lock.acquire()
		try:
			existing=self.assignedsockets[name]
		except KeyError:
			# this SHOULD happen. If not, something's wrong
			self.assignedsockets[name]=info
			self.lock.release()
			return 0 # Why is this, you get cannot marshal None unless allow_none is enabled without?!?!
		self.lock.release()
		raise Exception("key "+name+" already in use")
		
		
	def getServices(self):
		return self.assignedsockets
		
	#--------------------STATIC FUNCTIONS, CALLABLE FROM ANY MACHINE-----------------------
	#NOTE, STATIC FUNCTIONS HAVE NO ACCESS TO SELF

	
	 # returns pair (machienname, portnumber)
	 # TODO make this more flexible??
	def getInfoServerAddressAndPort():
		return ("urk.twi.tudelft.nl",8001)

	getInfoServerAddressAndPort=Callable(getInfoServerAddressAndPort)
	
	# convert (name,port) into "http://machinename:portnr"
	def makeFullName((name,port)):
		return "http://"+name+":"+str(port)
		
	makeFullName=Callable(makeFullName)
		
	
	def getInfoServerProxy():
		ad=NetworkTools.makeFullName(NetworkTools.getInfoServerAddressAndPort())
		return xmlrpclib.ServerProxy(ad)
	
	getInfoServerProxy=Callable(getInfoServerProxy)
	
	
	# getInfo() assumes that the Network tools server has been started on the appropriate machine.
	#  @return pair (webadres,socketnumber). returns -1 if the tools server is not running 
	# or there is no free socket.
	# @name is name of service, typically things like patientid.servicename
	# @param waitforit can be set to True if you want getSocket to wait for service to become available.
	# @param maxtries=max number of tries to get connected.
	
	# WARNING. if you try to call getInfo through XMLRPC you will get
	# xmlrpclib.Fault: <Fault 1: '<type \'exceptions.Exception\'>:method "getInfo" is not supported'>
	#
	def getInfo(name,waitforit, maxtries):
		# do not refer to self, callers will set it to 0 or so.
		print "looking up "+name
		proxy = NetworkTools.getInfoServerProxy()
		socket=proxy.getInfoNoWait(name)
		while (waitforit and socket==-1 and maxtries>0):
			time.sleep(1)
			socket=proxy.getInfoNoWait(name)
			maxtries=maxtries-1
		return socket
	
	getInfo=Callable(getInfo)
	